
/*==========================================================================*/
/*                                                                          */
/*       Program for the two-dimensional Ising model simulation             */
/*                                                                          */
/*       PROGRAM NAME:    ising.c                                           */
/*                                                                          */
/*       CALLING SEQUENCE:                                                  */
/*                                                                          */
/*         Main program without command line arguments.                     */
/*                                                                          */
/*       RETURN:                                                            */
/*                                                                          */
/*         Type int       - Retrun code                                     */
/*                                                                          */
/*   Version 1.00                                                           */
/*                                                                          */
/*   (c) Satoru Ozawa and Dieter W. Heermann                                */
/*   Hitachi, Heidelberg July 1994                                          */
/*                                                                          */
/*==========================================================================*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*  Subroutines required for this program:                                  */
/*                                                                          */
/*        ising2d    Ising model routine                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/

/* ==== Include files for this program ==== */

# include <math.h>
# include <stdio.h>
# include <stdlib.h>

# include "sgl.h"               /* Include for the graphics library   */
# include "ising2d.c"           /* Ising simulation program           */
# include "r250.c"              /* Shift-register generator           */

/*--------------------------------------------------------------------------*/
/*                         Main program                                     */
/*--------------------------------------------------------------------------*/
#ifdef  __STDC__
void    main(int argc, char *argv[])
#else
void    main(argc,argv)
        int     argc;
        char    *argv[];
#endif
  {
    /*---------------------------------------------------------- */
    /*              Declarations                                 */
    /*---------------------------------------------------------- */

    int     *lattice;
    float   *prob_distx, *prob_disty;
    float   *m_trace;

    int    i;
    int      L;
    float   ttc;
    float   h;
    int     max_sweeps;
    int     start;
    int     ana;
    int     seed;
    int     type;
    int    init;
    float  a_mag;
    float  sus;
    float  cum;

    int     return_code;
    int    size;

    float minimum, maximum;
    int   model;
    int   dist_n;


    /* ==== Pointer declarations ==== */

    FILE   *cid;
    FILE   *did;

    /*---------------------------------------------------------- */
    /*                End of declares                            */
    /*---------------------------------------------------------- */

    /*---------------------------------------------------------- */
    /*      Start with setting up procedures                     */
    /*---------------------------------------------------------- */

    /* ====  Print the header for this simulation  ==== */

    printf("===============================================================\n");
    printf("= > 2-Dimensional Ising Model Simulation with Applied Field < =\n");
    printf("===============================================================\n");
    printf("(c) Satoru Ozawa and Dieter W. Heermann \n");
    printf("Version 1.00 (1994) \n");
    printf("\n\n");

    printf("Linear system size                    :");
    scanf("%d",&L);
    printf("ttc for the simulation                :");
    scanf("%f",&ttc);
    printf("Applied magnetic field                :");
    scanf("%f",&h);
    printf("Number of sweeps for this run         :");
    scanf("%d",&max_sweeps);
    printf("Take measurements every               :");
    scanf("%d",&ana);
    printf("Begin measurements at                 :");
    scanf("%d",&start);
    printf("Seed for the random number generator  :");
    scanf("%d",&seed);
    printf("Type of analysis                      :");
    scanf("%d",&type);


    /* ==== Prepare by initializing all necessary quantities ==== */

    size    = L * L;
    lattice = (int *) calloc(size, sizeof(int));
    if ( lattice == NULL ) {
        exit(-1);
    }

    cid = fopen("trace","w");
    if ( cid == NULL ) {
        printf("Unable to open trace file\n");
        exit(100);
    }


    switch( type ) {
        case 1 :
            m_trace = (float *) calloc(max_sweeps, sizeof(float));
            if ( m_trace == NULL) {
              printf("Out of memory: exit code -1\n");
              exit(-1);
            }
            break;
        default :
            break;
    }

    init = 0;
    return_code = ising_2d_pbc(L, lattice, ttc, h, max_sweeps,
                               start, ana, seed, type, init,
                               a_mag, &sus, &cum, m_trace);

    printf("return code was = %d\n",return_code);

    if ( type == 1 ) {
        printf("write trace file \n");
        for(i=0;i < max_sweeps;i++) {
            fprintf(cid,"%d   %f\n",i,m_trace[i]);
        }
    }

    printf("Result:\n");
    printf("--------------------------\n");
    printf("Magnetization    = %f\n",a_mag);
    printf("Susceptibility   = %f\n",sus);
    printf("Cumulant         = %f\n",cum);
}
