/*=====================================================================*/
/*                                                                     */
/*       FILE NAME:        runge3.c                                    */
/*                                                                     */
/*       PROGRAM NAME:     runge3   (Prog. 2.3)                        */
/*                                                                     */
/*                                                                     */
/*   Version 1.00                                                      */
/*                                                                     */
/*   (c) Satoru Ozawa and Dieter W. Heermann                           */
/*   Hitachi, Heidelberg July 1994                                     */
/*                                                                     */
/*=====================================================================*/

/***********************************************************************/
/* one-dimensional harmonic oscillator, 3-particles                    */
/* Runge-Kutta method                                                  */
/***********************************************************************/

#include <stdio.h>
#include "sgl.h"                /* include files for graphics       */

#define  dt      0.05           /* time interval of an unit MD step */
#define  maxmds  1000           /* maximum number of MD steps       */
#define  m       1.             /* mass of particle                 */
#define  k       1.             /* force constant                   */
#define  r       0.05           /* radius of disc                   */

void	main(int argc, char *argv[])
{
  float   x1, x2, x3, x1new, x2new, x3new;
  float   v1, v2, v3, v1new, v2new, v3new;
  float   s1, ss1, sss1, ssss1;
  float   s2, ss2, sss2, ssss2;
  float   s3, ss3, sss3, ssss3;
  float   a1, aa1, aaa1, aaaa1;
  float   a2, aa2, aaa2, aaaa2;
  float   a3, aa3, aaa3, aaaa3;
  float   e; 
  int     mds, i;

  sgl_winsize(512,512);    /* display-size in pixels */
  sgl_init();              /* initialize graphics    */
 
  x1 = +1.0;               /* inital condition       */
  x2 = 0.;
  x3 = 0.;
  v1 = 0.;
  v2 = 0.;
  v3 = 0.;
 
  for (mds = 1; mds <= maxmds ; mds++){      /* integration of equation */ 
                                             /* of motion               */
    a1 =                  k/m * (x2-x1);
    a2 = -k/m * (x2-x1) + k/m * (x3-x2);
    a3 = -k/m * (x3-x2);
    s1 = v1;
    s2 = v2;
    s3 = v3;

    aa1 =  k/m * ((x2+s2*dt/2.)-(x1+s1*dt/2.));
    aa2 = -k/m * ((x2+s2*dt/2.)-(x1+s1*dt/2.)) 
          +k/m * ((x3+s3*dt/2.)-(x2+s2*dt/2.));
    aa3 = -k/m * ((x3+s3*dt/2.)-(x2+s2*dt/2.));
    ss1 = v1 + a1*dt/2.;
    ss2 = v2 + a2*dt/2.;
    ss3 = v3 + a3*dt/2.;

    aaa1 =  k/m * ((x2+ss2*dt/2.)-(x1+ss1*dt/2.));
    aaa2 = -k/m * ((x2+ss2*dt/2.)-(x1+ss1*dt/2.))
           +k/m * ((x3+ss3*dt/2.)-(x2+ss2*dt/2.));
    aaa3 = -k/m * ((x3+ss3*dt/2.)-(x2+ss2*dt/2.));
    sss1 = v1 + aa1*dt/2.;
    sss2 = v2 + aa2*dt/2.;
    sss3 = v3 + aa3*dt/2.;

    aaaa1 =  k/m * ((x2+sss2*dt)-(x1+sss1*dt));
    aaaa2 = -k/m * ((x2+sss2*dt)-(x1+sss1*dt))
            +k/m * ((x3+sss3*dt)-(x2+sss2*dt));
    aaaa3 = -k/m * ((x3+sss3*dt)-(x2+sss2*dt));
    ssss1 = v1 + aaa1*dt;
    ssss2 = v2 + aaa2*dt;
    ssss3 = v3 + aaa3*dt;

    x1new = x1 + (s1+2.*ss1+2.*sss1+ssss1)*dt/6.;
    x2new = x2 + (s2+2.*ss2+2.*sss2+ssss2)*dt/6.;
    x3new = x3 + (s3+2.*ss3+2.*sss3+ssss3)*dt/6.;
    v1new = v1 + (a1+2.*aa1+2.*aaa1+aaaa1)*dt/6.;
    v2new = v2 + (a2+2.*aa2+2.*aaa2+aaaa2)*dt/6.;
    v3new = v3 + (a3+2.*aa3+2.*aaa3+aaaa3)*dt/6.;

    x1 = x1new;
    x2 = x2new;
    x3 = x3new;
    v1 = v1new;
    v2 = v2new;
    v3 = v3new;
    
    e = m/2.*(v1*v1 + v2*v2 + v3*v3)                  /* total energy */
        +k/2.*(x1-x2)*(x1-x2) + k/2.*(x2-x3)*(x2-x3);
    printf("mds=%d, x1=%f, x2=%f, x3=%f, e=%f \n", mds, x1, x2, x3, e);

    sgl_disc(x1*0.1+0.25, 0.50, r);                 /* display motion */
    sgl_disc(x2*0.1+0.50, 0.50, r);
    sgl_disc(x3*0.1+0.75, 0.50, r);
    sgl_display();
  }
  i = getchar();                 /* hold picture until hit of any key */
}

